@@ -1,3 +1,6 @@
+// SPDX-License-Identifier: MIT
+// SPDX-FileCopyrightText: © 2015 LabStack LLC and Echo contributors
+
 package echo
 
 import (
@@ -5,6 +8,7 @@ import (
 	"encoding/json"
 	"encoding/xml"
 	"errors"
+	"fmt"
 	"io"
 	"mime/multipart"
 	"net/http"
@@ -19,91 +23,91 @@ import (
 	"github.com/stretchr/testify/assert"
 )
 
-type (
-	bindTestStruct struct {
-		I           int
-		PtrI        *int
-		I8          int8
-		PtrI8       *int8
-		I16         int16
-		PtrI16      *int16
-		I32         int32
-		PtrI32      *int32
-		I64         int64
-		PtrI64      *int64
-		UI          uint
-		PtrUI       *uint
-		UI8         uint8
-		PtrUI8      *uint8
-		UI16        uint16
-		PtrUI16     *uint16
-		UI32        uint32
-		PtrUI32     *uint32
-		UI64        uint64
-		PtrUI64     *uint64
-		B           bool
-		PtrB        *bool
-		F32         float32
-		PtrF32      *float32
-		F64         float64
-		PtrF64      *float64
-		S           string
-		PtrS        *string
-		cantSet     string
-		DoesntExist string
-		GoT         time.Time
-		GoTptr      *time.Time
-		T           Timestamp
-		Tptr        *Timestamp
-		SA          StringArray
-	}
-	bindTestStructWithTags struct {
-		I           int      `json:"I" form:"I"`
-		PtrI        *int     `json:"PtrI" form:"PtrI"`
-		I8          int8     `json:"I8" form:"I8"`
-		PtrI8       *int8    `json:"PtrI8" form:"PtrI8"`
-		I16         int16    `json:"I16" form:"I16"`
-		PtrI16      *int16   `json:"PtrI16" form:"PtrI16"`
-		I32         int32    `json:"I32" form:"I32"`
-		PtrI32      *int32   `json:"PtrI32" form:"PtrI32"`
-		I64         int64    `json:"I64" form:"I64"`
-		PtrI64      *int64   `json:"PtrI64" form:"PtrI64"`
-		UI          uint     `json:"UI" form:"UI"`
-		PtrUI       *uint    `json:"PtrUI" form:"PtrUI"`
-		UI8         uint8    `json:"UI8" form:"UI8"`
-		PtrUI8      *uint8   `json:"PtrUI8" form:"PtrUI8"`
-		UI16        uint16   `json:"UI16" form:"UI16"`
-		PtrUI16     *uint16  `json:"PtrUI16" form:"PtrUI16"`
-		UI32        uint32   `json:"UI32" form:"UI32"`
-		PtrUI32     *uint32  `json:"PtrUI32" form:"PtrUI32"`
-		UI64        uint64   `json:"UI64" form:"UI64"`
-		PtrUI64     *uint64  `json:"PtrUI64" form:"PtrUI64"`
-		B           bool     `json:"B" form:"B"`
-		PtrB        *bool    `json:"PtrB" form:"PtrB"`
-		F32         float32  `json:"F32" form:"F32"`
-		PtrF32      *float32 `json:"PtrF32" form:"PtrF32"`
-		F64         float64  `json:"F64" form:"F64"`
-		PtrF64      *float64 `json:"PtrF64" form:"PtrF64"`
-		S           string   `json:"S" form:"S"`
-		PtrS        *string  `json:"PtrS" form:"PtrS"`
-		cantSet     string
-		DoesntExist string      `json:"DoesntExist" form:"DoesntExist"`
-		GoT         time.Time   `json:"GoT" form:"GoT"`
-		GoTptr      *time.Time  `json:"GoTptr" form:"GoTptr"`
-		T           Timestamp   `json:"T" form:"T"`
-		Tptr        *Timestamp  `json:"Tptr" form:"Tptr"`
-		SA          StringArray `json:"SA" form:"SA"`
-	}
-	Timestamp   time.Time
-	TA          []Timestamp
-	StringArray []string
-	Struct      struct {
-		Foo string
-	}
-	Bar struct {
-		Baz int `json:"baz" query:"baz"`
-	}
-)
+type bindTestStruct struct {
+	I           int
+	PtrI        *int
+	I8          int8
+	PtrI8       *int8
+	I16         int16
+	PtrI16      *int16
+	I32         int32
+	PtrI32      *int32
+	I64         int64
+	PtrI64      *int64
+	UI          uint
+	PtrUI       *uint
+	UI8         uint8
+	PtrUI8      *uint8
+	UI16        uint16
+	PtrUI16     *uint16
+	UI32        uint32
+	PtrUI32     *uint32
+	UI64        uint64
+	PtrUI64     *uint64
+	B           bool
+	PtrB        *bool
+	F32         float32
+	PtrF32      *float32
+	F64         float64
+	PtrF64      *float64
+	S           string
+	PtrS        *string
+	cantSet     string
+	DoesntExist string
+	GoT         time.Time
+	GoTptr      *time.Time
+	T           Timestamp
+	Tptr        *Timestamp
+	SA          StringArray
+}
+
+type bindTestStructWithTags struct {
+	I           int      `json:"I" form:"I"`
+	PtrI        *int     `json:"PtrI" form:"PtrI"`
+	I8          int8     `json:"I8" form:"I8"`
+	PtrI8       *int8    `json:"PtrI8" form:"PtrI8"`
+	I16         int16    `json:"I16" form:"I16"`
+	PtrI16      *int16   `json:"PtrI16" form:"PtrI16"`
+	I32         int32    `json:"I32" form:"I32"`
+	PtrI32      *int32   `json:"PtrI32" form:"PtrI32"`
+	I64         int64    `json:"I64" form:"I64"`
+	PtrI64      *int64   `json:"PtrI64" form:"PtrI64"`
+	UI          uint     `json:"UI" form:"UI"`
+	PtrUI       *uint    `json:"PtrUI" form:"PtrUI"`
+	UI8         uint8    `json:"UI8" form:"UI8"`
+	PtrUI8      *uint8   `json:"PtrUI8" form:"PtrUI8"`
+	UI16        uint16   `json:"UI16" form:"UI16"`
+	PtrUI16     *uint16  `json:"PtrUI16" form:"PtrUI16"`
+	UI32        uint32   `json:"UI32" form:"UI32"`
+	PtrUI32     *uint32  `json:"PtrUI32" form:"PtrUI32"`
+	UI64        uint64   `json:"UI64" form:"UI64"`
+	PtrUI64     *uint64  `json:"PtrUI64" form:"PtrUI64"`
+	B           bool     `json:"B" form:"B"`
+	PtrB        *bool    `json:"PtrB" form:"PtrB"`
+	F32         float32  `json:"F32" form:"F32"`
+	PtrF32      *float32 `json:"PtrF32" form:"PtrF32"`
+	F64         float64  `json:"F64" form:"F64"`
+	PtrF64      *float64 `json:"PtrF64" form:"PtrF64"`
+	S           string   `json:"S" form:"S"`
+	PtrS        *string  `json:"PtrS" form:"PtrS"`
+	cantSet     string
+	DoesntExist string      `json:"DoesntExist" form:"DoesntExist"`
+	GoT         time.Time   `json:"GoT" form:"GoT"`
+	GoTptr      *time.Time  `json:"GoTptr" form:"GoTptr"`
+	T           Timestamp   `json:"T" form:"T"`
+	Tptr        *Timestamp  `json:"Tptr" form:"Tptr"`
+	SA          StringArray `json:"SA" form:"SA"`
+}
+
+type Timestamp time.Time
+type TA []Timestamp
+type StringArray []string
+type Struct struct {
+	Foo string
+}
+type Bar struct {
+	Baz int `json:"baz" query:"baz"`
+}
 
 func (t *Timestamp) UnmarshalParam(src string) error {
 	ts, err := time.Parse(time.RFC3339, src)
@@ -164,6 +168,11 @@ var values = map[string][]string{
 	"ST":      {"bar"},
 }
 
+// ptr return pointer to value. This is useful as `v := []*int8{&int8(1)}` will not compile
+func ptr[T any](value T) *T {
+	return &value
+}
+
 func TestToMultipleFields(t *testing.T) {
 	e := New()
 	req := httptest.NewRequest(http.MethodGet, "/?id=1&ID=2", nil)
@@ -429,6 +438,109 @@ func TestBindUnsupportedMediaType(t *testing.T) {
 	testBindError(t, strings.NewReader(invalidContent), MIMEApplicationJSON, &json.SyntaxError{})
 }
 
+func TestDefaultBinder_bindDataToMap(t *testing.T) {
+	exampleData := map[string][]string{
+		"multiple": {"1", "2"},
+		"single":   {"3"},
+	}
+
+	t.Run("ok, bind to map[string]string", func(t *testing.T) {
+		dest := map[string]string{}
+		assert.NoError(t, new(DefaultBinder).bindData(&dest, exampleData, "param"))
+		assert.Equal(t,
+			map[string]string{
+				"multiple": "1",
+				"single":   "3",
+			},
+			dest,
+		)
+	})
+
+	t.Run("ok, bind to map[string]string with nil map", func(t *testing.T) {
+		var dest map[string]string
+		assert.NoError(t, new(DefaultBinder).bindData(&dest, exampleData, "param"))
+		assert.Equal(t,
+			map[string]string{
+				"multiple": "1",
+				"single":   "3",
+			},
+			dest,
+		)
+	})
+
+	t.Run("ok, bind to map[string][]string", func(t *testing.T) {
+		dest := map[string][]string{}
+		assert.NoError(t, new(DefaultBinder).bindData(&dest, exampleData, "param"))
+		assert.Equal(t,
+			map[string][]string{
+				"multiple": {"1", "2"},
+				"single":   {"3"},
+			},
+			dest,
+		)
+	})
+
+	t.Run("ok, bind to map[string][]string with nil map", func(t *testing.T) {
+		var dest map[string][]string
+		assert.NoError(t, new(DefaultBinder).bindData(&dest, exampleData, "param"))
+		assert.Equal(t,
+			map[string][]string{
+				"multiple": {"1", "2"},
+				"single":   {"3"},
+			},
+			dest,
+		)
+	})
+
+	t.Run("ok, bind to map[string]interface", func(t *testing.T) {
+		dest := map[string]interface{}{}
+		assert.NoError(t, new(DefaultBinder).bindData(&dest, exampleData, "param"))
+		assert.Equal(t,
+			map[string]interface{}{
+				"multiple": []string{"1", "2"},
+				"single":   []string{"3"},
+			},
+			dest,
+		)
+	})
+
+	t.Run("ok, bind to map[string]interface with nil map", func(t *testing.T) {
+		var dest map[string]interface{}
+		assert.NoError(t, new(DefaultBinder).bindData(&dest, exampleData, "param"))
+		assert.Equal(t,
+			map[string]interface{}{
+				"multiple": []string{"1", "2"},
+				"single":   []string{"3"},
+			},
+			dest,
+		)
+	})
+
+	t.Run("ok, bind to map[string]int skips", func(t *testing.T) {
+		dest := map[string]int{}
+		assert.NoError(t, new(DefaultBinder).bindData(&dest, exampleData, "param"))
+		assert.Equal(t, map[string]int{}, dest)
+	})
+
+	t.Run("ok, bind to map[string]int skips with nil map", func(t *testing.T) {
+		var dest map[string]int
+		assert.NoError(t, new(DefaultBinder).bindData(&dest, exampleData, "param"))
+		assert.Equal(t, map[string]int(nil), dest)
+	})
+
+	t.Run("ok, bind to map[string][]int skips", func(t *testing.T) {
+		dest := map[string][]int{}
+		assert.NoError(t, new(DefaultBinder).bindData(&dest, exampleData, "param"))
+		assert.Equal(t, map[string][]int{}, dest)
+	})
+
+	t.Run("ok, bind to map[string][]int skips with nil map", func(t *testing.T) {
+		var dest map[string][]int
+		assert.NoError(t, new(DefaultBinder).bindData(&dest, exampleData, "param"))
+		assert.Equal(t, map[string][]int(nil), dest)
+	})
+}
+
 func TestBindbindData(t *testing.T) {
 	ts := new(bindTestStruct)
 	b := new(DefaultBinder)
@@ -547,49 +659,6 @@ func TestBindSetWithProperType(t *testing.T) {
 	assert.Error(t, setWithProperType(typ.Field(0).Type.Kind(), "5", val.Field(0)))
 }
 
-func TestBindSetFields(t *testing.T) {
-
-	ts := new(bindTestStruct)
-	val := reflect.ValueOf(ts).Elem()
-	// Int
-	if assert.NoError(t, setIntField("5", 0, val.FieldByName("I"))) {
-		assert.Equal(t, 5, ts.I)
-	}
-	if assert.NoError(t, setIntField("", 0, val.FieldByName("I"))) {
-		assert.Equal(t, 0, ts.I)
-	}
-
-	// Uint
-	if assert.NoError(t, setUintField("10", 0, val.FieldByName("UI"))) {
-		assert.Equal(t, uint(10), ts.UI)
-	}
-	if assert.NoError(t, setUintField("", 0, val.FieldByName("UI"))) {
-		assert.Equal(t, uint(0), ts.UI)
-	}
-
-	// Float
-	if assert.NoError(t, setFloatField("15.5", 0, val.FieldByName("F32"))) {
-		assert.Equal(t, float32(15.5), ts.F32)
-	}
-	if assert.NoError(t, setFloatField("", 0, val.FieldByName("F32"))) {
-		assert.Equal(t, float32(0.0), ts.F32)
-	}
-
-	// Bool
-	if assert.NoError(t, setBoolField("true", val.FieldByName("B"))) {
-		assert.Equal(t, true, ts.B)
-	}
-	if assert.NoError(t, setBoolField("", val.FieldByName("B"))) {
-		assert.Equal(t, false, ts.B)
-	}
-
-	ok, err := unmarshalFieldNonPtr("2016-12-06T19:09:05Z", val.FieldByName("T"))
-	if assert.NoError(t, err) {
-		assert.Equal(t, ok, true)
-		assert.Equal(t, Timestamp(time.Date(2016, 12, 6, 19, 9, 5, 0, time.UTC)), ts.T)
-	}
-}
-
 func BenchmarkBindbindDataWithTags(b *testing.B) {
 	b.ReportAllocs()
 	ts := new(bindTestStructWithTags)
@@ -683,7 +752,7 @@ func testBindError(t *testing.T, r io.Reader, ctype string, expectedInternal err
 }
 
 func TestDefaultBinder_BindToStructFromMixedSources(t *testing.T) {
-	// tests to check binding behaviour when multiple sources path params, query params and request body are in use
+	// tests to check binding behaviour when multiple sources (path params, query params and request body) are in use
 	// binding is done in steps and one source could overwrite previous source binded data
 	// these tests are to document this behaviour and detect further possible regressions when bind implementation is changed
 
@@ -853,7 +922,7 @@ func TestDefaultBinder_BindToStructFromMixedSources(t *testing.T) {
 }
 
 func TestDefaultBinder_BindBody(t *testing.T) {
-	// tests to check binding behaviour when multiple sources path params, query params and request body are in use
+	// tests to check binding behaviour when multiple sources (path params, query params and request body) are in use
 	// generally when binding from request body - URL and path params are ignored - unless form is being binded.
 	// these tests are to document this behaviour and detect further possible regressions when bind implementation is changed
 
@@ -1032,3 +1101,322 @@ func TestDefaultBinder_BindBody(t *testing.T) {
 		})
 	}
 }
+
+func testBindURL(queryString string, target any) error {
+	e := New()
+	req := httptest.NewRequest(http.MethodGet, queryString, nil)
+	rec := httptest.NewRecorder()
+	c := e.NewContext(req, rec)
+	return c.Bind(target)
+}
+
+type unixTimestamp struct {
+	Time time.Time
+}
+
+func (t *unixTimestamp) UnmarshalParam(param string) error {
+	n, err := strconv.ParseInt(param, 10, 64)
+	if err != nil {
+		return fmt.Errorf("'%s' is not an integer", param)
+	}
+	*t = unixTimestamp{Time: time.Unix(n, 0)}
+	return err
+}
+
+type IntArrayA []int
+
+// UnmarshalParam converts value to *Int64Slice.  This allows the API to accept
+// a comma-separated list of integers as a query parameter.
+func (i *IntArrayA) UnmarshalParam(value string) error {
+	var values = strings.Split(value, ",")
+	var numbers = make([]int, 0, len(values))
+
+	for _, v := range values {
+		n, err := strconv.ParseInt(v, 10, 64)
+		if err != nil {
+			return fmt.Errorf("'%s' is not an integer", v)
+		}
+
+		numbers = append(numbers, int(n))
+	}
+
+	*i = append(*i, numbers...)
+	return nil
+}
+
+func TestBindUnmarshalParamExtras(t *testing.T) {
+	// this test documents how bind handles `BindUnmarshaler` interface:
+	// NOTE: BindUnmarshaler chooses first input value to be bound.
+
+	t.Run("nok, unmarshalling fails", func(t *testing.T) {
+		result := struct {
+			V unixTimestamp `query:"t"`
+		}{}
+		err := testBindURL("/?t=xxxx", &result)
+
+		assert.EqualError(t, err, "code=400, message='xxxx' is not an integer, internal='xxxx' is not an integer")
+	})
+
+	t.Run("ok, target is struct", func(t *testing.T) {
+		result := struct {
+			V unixTimestamp `query:"t"`
+		}{}
+		err := testBindURL("/?t=1710095540&t=1710095541", &result)
+
+		assert.NoError(t, err)
+		expect := unixTimestamp{
+			Time: time.Unix(1710095540, 0),
+		}
+		assert.Equal(t, expect, result.V)
+	})
+
+	t.Run("ok, target is an alias to slice and is nil, append only values from first", func(t *testing.T) {
+		result := struct {
+			V IntArrayA `query:"a"`
+		}{}
+		err := testBindURL("/?a=1,2,3&a=4,5,6", &result)
+
+		assert.NoError(t, err)
+		assert.Equal(t, IntArrayA([]int{1, 2, 3}), result.V)
+	})
+
+	t.Run("ok, target is an alias to slice and is nil, single input", func(t *testing.T) {
+		result := struct {
+			V IntArrayA `query:"a"`
+		}{}
+		err := testBindURL("/?a=1,2", &result)
+
+		assert.NoError(t, err)
+		assert.Equal(t, IntArrayA([]int{1, 2}), result.V)
+	})
+
+	t.Run("ok, target is pointer an alias to slice and is nil", func(t *testing.T) {
+		result := struct {
+			V *IntArrayA `query:"a"`
+		}{}
+		err := testBindURL("/?a=1&a=4,5,6", &result)
+
+		assert.NoError(t, err)
+		var expected = IntArrayA([]int{1})
+		assert.Equal(t, &expected, result.V)
+	})
+
+	t.Run("ok, target is pointer an alias to slice and is NOT nil", func(t *testing.T) {
+		result := struct {
+			V *IntArrayA `query:"a"`
+		}{}
+		result.V = new(IntArrayA) // NOT nil
+
+		err := testBindURL("/?a=1&a=4,5,6", &result)
+
+		assert.NoError(t, err)
+		var expected = IntArrayA([]int{1})
+		assert.Equal(t, &expected, result.V)
+	})
+}
+
+type unixTimestampLast struct {
+	Time time.Time
+}
+
+// this is silly example for `bindMultipleUnmarshaler` for type that uses last input value for unmarshalling
+func (t *unixTimestampLast) UnmarshalParams(params []string) error {
+	lastInput := params[len(params)-1]
+	n, err := strconv.ParseInt(lastInput, 10, 64)
+	if err != nil {
+		return fmt.Errorf("'%s' is not an integer", lastInput)
+	}
+	*t = unixTimestampLast{Time: time.Unix(n, 0)}
+	return err
+}
+
+type IntArrayB []int
+
+func (i *IntArrayB) UnmarshalParams(params []string) error {
+	var numbers = make([]int, 0, len(params))
+
+	for _, param := range params {
+		var values = strings.Split(param, ",")
+		for _, v := range values {
+			n, err := strconv.ParseInt(v, 10, 64)
+			if err != nil {
+				return fmt.Errorf("'%s' is not an integer", v)
+			}
+			numbers = append(numbers, int(n))
+		}
+	}
+
+	*i = append(*i, numbers...)
+	return nil
+}
+
+func TestBindUnmarshalParams(t *testing.T) {
+	// this test documents how bind handles `bindMultipleUnmarshaler` interface:
+
+	t.Run("nok, unmarshalling fails", func(t *testing.T) {
+		result := struct {
+			V unixTimestampLast `query:"t"`
+		}{}
+		err := testBindURL("/?t=xxxx", &result)
+
+		assert.EqualError(t, err, "code=400, message='xxxx' is not an integer, internal='xxxx' is not an integer")
+	})
+
+	t.Run("ok, target is struct", func(t *testing.T) {
+		result := struct {
+			V unixTimestampLast `query:"t"`
+		}{}
+		err := testBindURL("/?t=1710095540&t=1710095541", &result)
+
+		assert.NoError(t, err)
+		expect := unixTimestampLast{
+			Time: time.Unix(1710095541, 0),
+		}
+		assert.Equal(t, expect, result.V)
+	})
+
+	t.Run("ok, target is an alias to slice and is nil, append multiple inputs", func(t *testing.T) {
+		result := struct {
+			V IntArrayB `query:"a"`
+		}{}
+		err := testBindURL("/?a=1,2,3&a=4,5,6", &result)
+
+		assert.NoError(t, err)
+		assert.Equal(t, IntArrayB([]int{1, 2, 3, 4, 5, 6}), result.V)
+	})
+
+	t.Run("ok, target is an alias to slice and is nil, single input", func(t *testing.T) {
+		result := struct {
+			V IntArrayB `query:"a"`
+		}{}
+		err := testBindURL("/?a=1,2", &result)
+
+		assert.NoError(t, err)
+		assert.Equal(t, IntArrayB([]int{1, 2}), result.V)
+	})
+
+	t.Run("ok, target is pointer an alias to slice and is nil", func(t *testing.T) {
+		result := struct {
+			V *IntArrayB `query:"a"`
+		}{}
+		err := testBindURL("/?a=1&a=4,5,6", &result)
+
+		assert.NoError(t, err)
+		var expected = IntArrayB([]int{1, 4, 5, 6})
+		assert.Equal(t, &expected, result.V)
+	})
+
+	t.Run("ok, target is pointer an alias to slice and is NOT nil", func(t *testing.T) {
+		result := struct {
+			V *IntArrayB `query:"a"`
+		}{}
+		result.V = new(IntArrayB) // NOT nil
+
+		err := testBindURL("/?a=1&a=4,5,6", &result)
+		assert.NoError(t, err)
+		var expected = IntArrayB([]int{1, 4, 5, 6})
+		assert.Equal(t, &expected, result.V)
+	})
+}
+
+func TestBindInt8(t *testing.T) {
+	t.Run("nok, binding fails", func(t *testing.T) {
+		type target struct {
+			V int8 `query:"v"`
+		}
+		p := target{}
+		err := testBindURL("/?v=x&v=2", &p)
+		assert.EqualError(t, err, "code=400, message=strconv.ParseInt: parsing \"x\": invalid syntax, internal=strconv.ParseInt: parsing \"x\": invalid syntax")
+	})
+
+	t.Run("nok, int8 embedded in struct", func(t *testing.T) {
+		type target struct {
+			int8 `query:"v"` // embedded field is `Anonymous`. We can only set public fields
+		}
+		p := target{}
+		err := testBindURL("/?v=1&v=2", &p)
+		assert.NoError(t, err)
+		assert.Equal(t, target{0}, p)
+	})
+
+	t.Run("nok, pointer to int8 embedded in struct", func(t *testing.T) {
+		type target struct {
+			*int8 `query:"v"` // embedded field is `Anonymous`. We can only set public fields
+		}
+		p := target{}
+		err := testBindURL("/?v=1&v=2", &p)
+		assert.NoError(t, err)
+
+		assert.Equal(t, target{int8: nil}, p)
+	})
+
+	t.Run("ok, bind int8 as struct field", func(t *testing.T) {
+		type target struct {
+			V int8 `query:"v"`
+		}
+		p := target{V: 127}
+		err := testBindURL("/?v=1&v=2", &p)
+		assert.NoError(t, err)
+		assert.Equal(t, target{V: 1}, p)
+	})
+
+	t.Run("ok, bind pointer to int8 as struct field, value is nil", func(t *testing.T) {
+		type target struct {
+			V *int8 `query:"v"`
+		}
+		p := target{}
+		err := testBindURL("/?v=1&v=2", &p)
+		assert.NoError(t, err)
+		assert.Equal(t, target{V: ptr(int8(1))}, p)
+	})
+
+	t.Run("ok, bind pointer to int8 as struct field, value is set", func(t *testing.T) {
+		type target struct {
+			V *int8 `query:"v"`
+		}
+		p := target{V: ptr(int8(127))}
+		err := testBindURL("/?v=1&v=2", &p)
+		assert.NoError(t, err)
+		assert.Equal(t, target{V: ptr(int8(1))}, p)
+	})
+
+	t.Run("ok, bind int8 slice as struct field, value is nil", func(t *testing.T) {
+		type target struct {
+			V []int8 `query:"v"`
+		}
+		p := target{}
+		err := testBindURL("/?v=1&v=2", &p)
+		assert.NoError(t, err)
+		assert.Equal(t, target{V: []int8{1, 2}}, p)
+	})
+
+	t.Run("ok, bind slice of int8 as struct field, value is set", func(t *testing.T) {
+		type target struct {
+			V []int8 `query:"v"`
+		}
+		p := target{V: []int8{111}}
+		err := testBindURL("/?v=1&v=2", &p)
+		assert.NoError(t, err)
+		assert.Equal(t, target{V: []int8{1, 2}}, p)
+	})
+
+	t.Run("ok, bind slice of pointer to int8 as struct field, value is set", func(t *testing.T) {
+		type target struct {
+			V []*int8 `query:"v"`
+		}
+		p := target{V: []*int8{ptr(int8(127))}}
+		err := testBindURL("/?v=1&v=2", &p)
+		assert.NoError(t, err)
+		assert.Equal(t, target{V: []*int8{ptr(int8(1)), ptr(int8(2))}}, p)
+	})
+
+	t.Run("ok, bind pointer to slice of int8 as struct field, value is set", func(t *testing.T) {
+		type target struct {
+			V *[]int8 `query:"v"`
+		}
+		p := target{V: &[]int8{111}}
+		err := testBindURL("/?v=1&v=2", &p)
+		assert.NoError(t, err)
+		assert.Equal(t, target{V: &[]int8{1, 2}}, p)
+	})
+}
